@@ -12,8 +12,8 @@ package Net::BitTorrent::DHT;
     use Net::BitTorrent::Protocol qw[:dht];
     use Net::BitTorrent::Version;
     use version qw[qv];
-    our $SVN = q[$Id: DHT.pm 46 2008-12-30 23:25:17Z sanko@cpan.org $];
-    our $UNSTABLE_RELEASE = 0; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new((qw$Rev: 46 $)[1])->numify / 1000), $UNSTABLE_RELEASE);
+    our $SVN = q[$Id: DHT.pm 49 2009-01-05 22:38:02Z sanko@cpan.org $];
+    our $UNSTABLE_RELEASE = 1; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new((qw$Rev: 49 $)[1])->numify / 1000), $UNSTABLE_RELEASE);
     my @CONTENTS
         = \my (%_client, %tid, %node_id, %outstanding_p, %nodes, %tracking);
     my %REGISTRY;
@@ -72,6 +72,13 @@ package Net::BitTorrent::DHT;
         return $_client{refaddr + $_[0]};
     }
 
+    sub _peers {
+        my ($self, $info_hash) = @_;
+        return q[] if !$tracking{refaddr $self}{$info_hash};
+        $tracking{refaddr $self}{$info_hash}{q[touch]} = time;
+        return $tracking{refaddr $self}{$info_hash}{q[peers]};
+    }
+
     # Accesors | Public
     sub node_id {
         return if defined $_[1];
@@ -417,17 +424,19 @@ package Net::BitTorrent::DHT;
                                             ->{q[a]}{q[info_hash]}
                         );
                     if ($torrent) {
-                        for my $value (@{$packet->{q[r]}{q[values]}}) {
-
-                            #warn q[********** add to torrent: ]
-                            #    . uncompact($value);
-                            $torrent->_append_nodes($value);
-                        }
-                    }
-                    else {
-
-                        #
-                        # xxx - ...what to do? What... to... do...
+                        $tracking{refaddr $self}{$torrent->infohash} = {
+                                    peers =>
+                                        compact(
+                                        uncompact(
+                                            $tracking{refaddr $self}
+                                                {$torrent->infohash}{q[peers]}
+                                        ),
+                                        (map { uncompact($_) }
+                                             @{$packet->{q[r]}{q[values]}}
+                                        )
+                                        ),
+                                    touch => time
+                        };
                     }
                     $self->_find_node_out($node, $node_id{refaddr $self});
                 }
@@ -725,16 +734,8 @@ specification.  These will all be fixed in a future version.
 
 =item *
 
-Eats memory like whoa.
-
-=item *
-
 The routing table is flat.
 
-=item *
-
-Boots from router.bittorrent.com.
-
 =back
 
 =head1 Notes
@@ -790,6 +791,6 @@ clarification, see http://creativecommons.org/licenses/by-sa/3.0/us/.
 Neither this module nor the L<Author|/Author> is affiliated with
 BitTorrent, Inc.
 
-=for svn $Id: DHT.pm 46 2008-12-30 23:25:17Z sanko@cpan.org $
+=for svn $Id: DHT.pm 49 2009-01-05 22:38:02Z sanko@cpan.org $
 
 =cut
