@@ -24,15 +24,15 @@ package Net::BitTorrent::Torrent;
     use Net::BitTorrent::Torrent::File;
     use Net::BitTorrent::Torrent::Tracker;
     use version qw[qv];
-    our $SVN = q[$Id: Torrent.pm 46 2008-12-30 23:25:17Z sanko@cpan.org $];
-    our $UNSTABLE_RELEASE = 0; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new((qw$Rev: 46 $)[1])->numify / 1000), $UNSTABLE_RELEASE);
+    our $SVN = q[$Id: Torrent.pm 49 2009-01-05 22:38:02Z sanko@cpan.org $];
+    our $UNSTABLE_RELEASE = 3; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new((qw$Rev: 49 $)[1])->numify / 1000), $UNSTABLE_RELEASE);
     my %REGISTRY = ();
-    my @CONTENTS = \my (%_client,       %path,     %_basedir,
-                        %size,          %files,    %trackers,
-                        %infohash,      %uploaded, %downloaded,
-                        %_nodes,        %bitfield, %_working_pieces,
-                        %_block_length, %raw_data, %status,
-                        %error,         %_event
+    my @CONTENTS = \my (%_client,  %path,            %_basedir,
+                        %size,     %files,           %trackers,
+                        %infohash, %uploaded,        %downloaded,
+                        %bitfield, %_working_pieces, %_block_length,
+                        %raw_data, %status,          %error,
+                        %_event,   %resume_path,     %_nodes
     );
     sub STARTED           {1}
     sub CHECKING          {2}
@@ -81,7 +81,7 @@ package Net::BitTorrent::Torrent;
         }
         if ($args->{q[Status]} and $args->{q[Status]} !~ m[^\d+$]) {
             carp q[Net::BitTorrent::Torrent->new({ }) requires an ]
-                . q[integer 'Status' parameter];
+                . q[integer 'Status' parameter.  Falling back to defaults.];
             delete $args->{q[Status]};
         }
         $args->{q[Path]} = rel2abs($args->{q[Path]});
@@ -124,10 +124,6 @@ package Net::BitTorrent::Torrent;
             )
         {   return;
         }
-        if (defined $args->{q[Client]}) {
-            $_client{refaddr $self} = $args->{q[Client]};
-            weaken $_client{refaddr $self};
-        }
         $infohash{refaddr $self}
             = sha1_hex(bencode($raw_data{refaddr $self}{q[info]}));
         $path{refaddr $self}            = $args->{q[Path]};
@@ -142,21 +138,8 @@ package Net::BitTorrent::Torrent;
         $_nodes{refaddr $self}     = q[];
         ${$bitfield{refaddr $self}}
             = pack(q[b*], qq[\0] x $self->piece_count);
-
-        if (defined $args->{q[Status]}) {
-            $args->{q[Status]} ^= LOADED if $args->{q[Status]} & LOADED;
-            $args->{q[Status]} ^= QUEUED if $args->{q[Status]} & QUEUED;
-        }
-        ${$status{refaddr $self}} |= (
-                               defined $args->{q[Status]} ? $args->{q[Status]}
-                               : defined $_client{refaddr $self} ? 1
-                               : 0
-        );
-        ${$status{refaddr $self}} |= LOADED;
-        ${$status{refaddr $self}} |= QUEUED
-            if defined $_client{refaddr $self};
-        ${$error{refaddr $self}} = undef;
         my @_files;
+
         if (defined $raw_data{refaddr $self}{q[info]}{q[files]}) {
             for my $file (@{$raw_data{refaddr $self}{q[info]}{q[files]}}) {
                 push @_files,
@@ -217,87 +200,100 @@ package Net::BitTorrent::Torrent;
                  )
             );
         }
-        if ($_client{refaddr $self}) {
-            $_client{refaddr $self}->_schedule(
-                                     {Time   => time + 25,
-                                      Code   => sub { shift->_dht_announce },
-                                      Object => $self
-                                     }
-            );
-            $_client{refaddr $self}->_schedule(
-                                       {Time   => time + 15,
-                                        Code   => sub { shift->_dht_scrape },
-                                        Object => $self
-                                       }
-            );
-        }
+        $args->{q[Status]} ||= 0;
+        $args->{q[Status]} ^= CHECKING if $args->{q[Status]} & CHECKING;
+        $args->{q[Status]} ^= CHECKED  if $args->{q[Status]} & CHECKED;
+        $args->{q[Status]} ^= ERROR    if $args->{q[Status]} & ERROR;
+        $args->{q[Status]} ^= LOADED   if $args->{q[Status]} & LOADED;
+        ${$status{refaddr $self}} = $args->{q[Status]};
+        ${$status{refaddr $self}} |= LOADED;
+        ${$error{refaddr $self}} = undef;
 
-        # Resume system
-        if (   $raw_data{refaddr $self}{q[net-bittorrent]}
-            && $raw_data{refaddr $self}{q[net-bittorrent]}{q[.version]}
-            && $raw_data{refaddr $self}{q[net-bittorrent]}{q[.version]}
-            <= 1    # apiver
-            )
-        {   $_nodes{refaddr $self}
-                = $raw_data{refaddr $self}{q[net-bittorrent]}{q[nodes]};
-            my $_okay = 1;
-            for my $_index (0 .. $#{$files{refaddr $self}}) {
-                if ((!-f $files{refaddr $self}->[$_index]->path
-                     && $raw_data{refaddr $self}{q[net-bittorrent]}{q[files]}
-                     [$_index]{q[mtime]}
-                    )
-                    || ((stat($files{refaddr $self}->[$_index]->path))[9]
-                        || 0 != $raw_data{refaddr $self}{q[net-bittorrent]}
-                        {q[files]}[$_index]{q[mtime]})
-                    )
-                {   ${$status{refaddr $self}} |= START_AFTER_CHECK;
-                    $self->_set_error(q[Bad resume data. Please hashcheck.]);
-                    $_okay = 0;
-                }
-                $files{refaddr $self}->[$_index]->set_priority(
-                         $raw_data{refaddr $self}{q[net-bittorrent]}{q[files]}
-                             [$_index]{q[priority]});
+        # Resume system v2
+        my $_start = 1;
+        $resume_path{refaddr $self} = undef;
+        if ($args->{q[Resume]}) {
+            $resume_path{refaddr $self} = $args->{q[Resume]};
+            my $_resume_data;
+            if (-f $args->{q[Resume]}) {
+                open(my ($_RD), q[<], $resume_path{refaddr $self});
+                sysread($_RD, $_resume_data, -s $_RD);
+                close $_RD;
             }
-            if ($_okay) {
-                ${$bitfield{refaddr $self}}
-                    = $raw_data{refaddr $self}{q[net-bittorrent]}
-                    {q[bitfield]};
+            if ($_resume_data) {
+                $_start       = 0;
+                $_resume_data = bdecode($_resume_data);
 
-                # Accept resume data is the same as hashchecking
-                my $start_after_check = (
-                         ((${$status{refaddr $self}} & QUEUED)
-                              && ${$status{refaddr $self}} & START_AFTER_CHECK
-                         )
-                             || ${$status{refaddr $self}} & STARTED
-                );
-                ${$status{refaddr $self}} ^= START_AFTER_CHECK
-                    if ${$status{refaddr $self}} & START_AFTER_CHECK;
-                ${$status{refaddr $self}} ^= CHECKED
-                    if !(${$status{refaddr $self}} & CHECKED);
-                if ($start_after_check) { $self->start(); }
-
-                # Reload Blocks
-                for my $_piece (
-                    @{$raw_data{refaddr $self}{q[net-bittorrent]}{q[working]}}
+                # Resume system
+                if (   $_resume_data->{q[.format]}
+                    && $_resume_data->{q[.format]} eq
+                    q[Net::BitTorrent resume]
+                    && $_resume_data->{q[.version]}
+                    && $_resume_data->{q[.version]} <= 2    # apiver
                     )
-                {   $_working_pieces{refaddr $self}{$_piece->{q[Index]}} = {
-                        Index    => $_piece->{q[Index]},
-                        Priority => $_piece->{q[Priority]},
-                        Blocks_Requested =>
-                            [map { {} } 1 .. $_piece->{q[Block_Count]}],
-                        Blocks_Received => [
-                            map {
-                                vec($_piece->{q[Blocks_Received]}, $_, 1)
-                                } 1 .. $_piece->{q[Block_Count]}
-                        ],
-                        Block_Length      => $_piece->{q[Block_Length]},
-                        Block_Length_Last => $_piece->{q[Block_Length_Last]},
-                        Block_Count       => $_piece->{q[Block_Count]},
-                        Length            => $_piece->{q[Length]},
-                        Endgame           => $_piece->{q[Endgame]},
-                        Slow  => 1,     # $_piece->{q[Slow]},
-                        mtime => time
-                    };
+                {   $_nodes{refaddr $self}
+                        = $_resume_data->{q[peers]}
+                        ? $_resume_data->{q[peers]}
+                        : q[];
+                    my $_okay = 1;
+                    for my $_index (0 .. $#{$files{refaddr $self}}) {
+                        if ((!-f $files{refaddr $self}->[$_index]->path
+                             && $_resume_data->{q[files]}[$_index]{q[mtime]}
+                            )
+                            || ((stat($files{refaddr $self}->[$_index]->path))
+                                [9]
+                                || 0 != $_resume_data->{q[files]}[$_index]
+                                {q[mtime]})
+                            )
+                        {   ${$status{refaddr $self}} |= START_AFTER_CHECK;
+                            $_okay = 0;
+                        }
+                        $files{refaddr $self}->[$_index]->set_priority(
+                             $_resume_data->{q[files]}[$_index]{q[priority]});
+                    }
+                    if (!$_okay) {
+                        $self->_set_error(
+                                       q[Bad resume data. Please hashcheck.]);
+                    }
+                    else {
+                        ${$bitfield{refaddr $self}}
+                            = $_resume_data->{q[bitfield]};
+
+                        # Accept resume data is the same as hashchecking
+                        my $start_after_check
+                            = ${$status{refaddr $self}} & START_AFTER_CHECK;
+                        ${$status{refaddr $self}} ^= START_AFTER_CHECK
+                            if ${$status{refaddr $self}} & START_AFTER_CHECK;
+                        ${$status{refaddr $self}} ^= CHECKED
+                            if !(${$status{refaddr $self}} & CHECKED);
+                        if ($start_after_check) { $_start = 1; }
+
+                        # Reload Blocks
+                        for my $_piece (@{$_resume_data->{q[working]}}) {
+                            $_working_pieces{refaddr $self}
+                                {$_piece->{q[Index]}} = {
+                                Index            => $_piece->{q[Index]},
+                                Priority         => $_piece->{q[Priority]},
+                                Blocks_Requested => [
+                                     map { {} } 1 .. $_piece->{q[Block_Count]}
+                                ],
+                                Blocks_Received => [
+                                    map {
+                                        vec($_piece->{q[Blocks_Received]},
+                                            $_, 1)
+                                        } 1 .. $_piece->{q[Block_Count]}
+                                ],
+                                Block_Length => $_piece->{q[Block_Length]},
+                                Block_Length_Last =>
+                                    $_piece->{q[Block_Length_Last]},
+                                Block_Count => $_piece->{q[Block_Count]},
+                                Length      => $_piece->{q[Length]},
+                                Endgame     => $_piece->{q[Endgame]},
+                                Slow  => 1,     # $_piece->{q[Slow]},
+                                mtime => time
+                                };
+                        }
+                    }
                 }
             }
         }
@@ -310,23 +306,40 @@ package Net::BitTorrent::Torrent;
             threads::shared::share($error{refaddr $self});
         }
         $$self = $infohash{refaddr $self};
-        $self->start if ${$status{refaddr $self}} & STARTED;
+        if ($args->{q[Client]}) {
+            $self->queue($args->{q[Client]});
+            $_client{refaddr $self}->_schedule(
+                                     {Time   => time + 25,
+                                      Code   => sub { shift->_dht_announce },
+                                      Object => $self
+                                     }
+            );
+            $_client{refaddr $self}->_schedule(
+                                       {Time   => time,
+                                        Code   => sub { shift->_dht_scrape },
+                                        Object => $self
+                                       }
+            );
+        }
+        $self->start if $_start && (${$status{refaddr $self}} & QUEUED);
+        $self->_new_peer();    # XXX - temporary multi-thread vs schedule fix
         return $self;
     }
 
     # Accessors | Public
-    sub infohash   { return $infohash{refaddr +shift}; }
-    sub trackers   { return $trackers{refaddr +shift}; }
-    sub bitfield   { return ${$bitfield{refaddr +shift}}; }
-    sub path       { return $path{refaddr +shift}; }
-    sub files      { return $files{refaddr +shift}; }
-    sub size       { return $size{refaddr +shift}; }
-    sub status     { return ${$status{refaddr +shift}}; }
-    sub downloaded { return $downloaded{refaddr +shift}; }
-    sub uploaded   { return $uploaded{refaddr +shift}; }
-    sub error      { return ${$error{refaddr +shift}}; }
-    sub comment    { return $raw_data{refaddr +shift}{q[comment]}; }
-    sub created_by { return $raw_data{refaddr +shift}{q[created by]}; }
+    sub infohash    { return $infohash{refaddr +shift}; }
+    sub trackers    { return $trackers{refaddr +shift}; }
+    sub bitfield    { return ${$bitfield{refaddr +shift}}; }
+    sub path        { return $path{refaddr +shift}; }
+    sub resume_path { return $resume_path{refaddr +shift}; }
+    sub files       { return $files{refaddr +shift}; }
+    sub size        { return $size{refaddr +shift}; }
+    sub status      { return ${$status{refaddr +shift}}; }
+    sub downloaded  { return $downloaded{refaddr +shift}; }
+    sub uploaded    { return $uploaded{refaddr +shift}; }
+    sub error       { return ${$error{refaddr +shift}}; }
+    sub comment     { return $raw_data{refaddr +shift}{q[comment]}; }
+    sub created_by  { return $raw_data{refaddr +shift}{q[created by]}; }
 
     sub creation_date {
         return $raw_data{refaddr +shift}{q[creation date]};
@@ -346,7 +359,7 @@ package Net::BitTorrent::Torrent;
 
     sub is_complete {
         my ($self) = @_;
-        return if ${$status{refaddr $self}} & CHECKING;
+        return if (${$status{refaddr $self}} & CHECKING);
         return unpack(q[b*], $self->_wanted) !~ m[1] ? 1 : 0;
     }
 
@@ -360,10 +373,24 @@ package Net::BitTorrent::Torrent;
             );
     }
 
+    sub peers {
+        my ($self) = @_;
+        return if (${$status{refaddr $self}} & CHECKING);
+        return if !(${$status{refaddr $self}} & QUEUED);
+        my $_connections = $_client{refaddr $self}->_connections;
+        return map {
+            (    ($_->{q[Object]}->isa(q[Net::BitTorrent::Peer]))
+             and ($_->{q[Object]}->_torrent)
+             and ($_->{q[Object]}->_torrent eq $self))
+                ? $_->{q[Object]}
+                : ()
+        } values %$_connections;
+    }
+
     # Mutators | Private
     sub _set_bitfield {
         my ($self, $new_value) = @_;
-        return if ${$status{refaddr $self}} & CHECKING;
+        return if (${$status{refaddr $self}} & CHECKING);
         return if length ${$bitfield{refaddr $self}} != length $new_value;
 
         # XXX - make sure bitfield conforms to what we expect it to be
@@ -372,7 +399,7 @@ package Net::BitTorrent::Torrent;
 
     sub _set_status {
         my ($self, $new_value) = @_;
-        return if ${$status{refaddr $self}} & CHECKING;
+        return if (${$status{refaddr $self}} & CHECKING);
 
         # XXX - make sure status conforms to what we expect it to be
         return ${$status{refaddr $self}} = $new_value;
@@ -381,7 +408,7 @@ package Net::BitTorrent::Torrent;
     sub _set_error {
         my ($self, $msg) = @_;
         ${$error{refaddr $self}} = $msg;
-        $self->stop();
+        $self->stop() if ${$status{refaddr $self}} & STARTED;
         ${$status{refaddr $self}} |= ERROR;
         return 1;
     }
@@ -393,9 +420,10 @@ package Net::BitTorrent::Torrent;
     }
 
     # Accessors | Private
-    sub _client         { return $_client{refaddr +shift}; }
-    sub _block_length   { return $_block_length{refaddr +shift} }
-    sub _nodes          { return $_nodes{refaddr +shift}; }
+    sub _client       { return $_client{refaddr +shift}; }
+    sub _block_length { return $_block_length{refaddr +shift} }
+
+    #sub _nodes          { return $_nodes{refaddr +shift}; }
     sub _working_pieces { return $_working_pieces{refaddr +shift}; }
     sub _basedir        { return $_basedir{refaddr +shift}; }
 
@@ -441,22 +469,17 @@ package Net::BitTorrent::Torrent;
     # Methods | Public
     sub hashcheck {
         my ($self) = @_;
-        return if ${$status{refaddr $self}} & PAUSED;
+        return if (${$status{refaddr $self}} & PAUSED);
+        return if (${$status{refaddr $self}} & CHECKING);
         ${$bitfield{refaddr $self}}    # empty it first
             = pack(q[b*], qq[\0] x $self->piece_count);
-        my $start_after_check = (
-                         ((${$status{refaddr $self}} & QUEUED)
-                              && ${$status{refaddr $self}} & START_AFTER_CHECK
-                         )
-                             || ${$status{refaddr $self}} & STARTED
-        );
+        my $start_after_check = ${$status{refaddr $self}} & START_AFTER_CHECK;
         ${$status{refaddr $self}} |= CHECKING
             if !${$status{refaddr $self}} & CHECKING;
-        $self->stop();
         for my $index (0 .. ($self->piece_count - 1)) {
             $self->_check_piece_by_index($index);
         }
-        ${$status{refaddr $self}} ^= START_AFTER_CHECK
+        (${$status{refaddr $self}} ^= START_AFTER_CHECK)
             if ${$status{refaddr $self}} & START_AFTER_CHECK;
         ${$status{refaddr $self}} ^= CHECKED
             if !(${$status{refaddr $self}} & CHECKED);
@@ -481,37 +504,33 @@ package Net::BitTorrent::Torrent;
 
     sub start {
         my ($self) = @_;
-        if (!${$status{refaddr $self}} & QUEUED) {
-            carp q[Cannot start an orphan torrent];
-            return;
-        }
+        return if !(${$status{refaddr $self}} & QUEUED);
         ${$status{refaddr $self}} ^= ERROR
             if ${$status{refaddr $self}} & ERROR;
         ${$status{refaddr $self}} ^= PAUSED
             if ${$status{refaddr $self}} & PAUSED;
-        ${$status{refaddr $self}} |= STARTED
-            if !(${$status{refaddr $self}} & STARTED);
-        $_client{refaddr $self}->_schedule(
-                                  {Time   => time + 5,
-                                   Code   => sub { shift->_new_peer if @_; },
-                                   Object => $self
-                                  }
-        ) if defined $_client{refaddr $self};
+        if (!(${$status{refaddr $self}} & STARTED)) {
+            ${$status{refaddr $self}} |= STARTED;
+            for my $tracker (@{$trackers{refaddr $self}}) {
+                $tracker->_announce(q[started]);
+            }
+        }
         return ${$status{refaddr $self}};
     }
 
     sub stop {
         my ($self) = @_;
-        if (!${$status{refaddr $self}} & QUEUED) {
-            carp q[Cannot stop an orphan torrent];
-            return;
-        }
-        for my $_peer ($self->_peers) {
+        return if !(${$status{refaddr $self}} & QUEUED);
+        for my $_peer ($self->peers) {
             $_peer->_disconnect(q[Torrent has been stopped]);
         }
         for my $_file (@{$files{refaddr $self}}) { $_file->_close(); }
-        ${$status{refaddr $self}} ^= STARTED
-            if (${$status{refaddr $self}} & STARTED);
+        if (${$status{refaddr $self}} & STARTED) {
+            ${$status{refaddr $self}} ^= STARTED;
+            for my $tracker (@{$trackers{refaddr $self}}) {
+                $tracker->_announce(q[stopped]);
+            }
+        }
         return !!${$status{refaddr $self}} & STARTED;
     }
 
@@ -531,116 +550,115 @@ package Net::BitTorrent::Torrent;
         $_client{refaddr $self} = $client;
         weaken $_client{refaddr $self};
         ${$status{refaddr $self}} ^= QUEUED;
+
+        #$self->_new_peer();
         return $_client{refaddr $self};
     }
 
     # Methods | Private
     sub _add_uploaded {
         my ($self, $amount) = @_;
-        if (!${$status{refaddr $self}} & QUEUED) { return; }
-        return if not defined $_client{refaddr $self};
-        return if ${$status{refaddr $self}} & CHECKING;
+        return if (${$status{refaddr $self}} & CHECKING);
+        return if !(${$status{refaddr $self}} & QUEUED);
         return if not $amount;
         $uploaded{refaddr $self} += (($amount =~ m[^\d+$]) ? $amount : 0);
     }
 
     sub _add_downloaded {
         my ($self, $amount) = @_;
-        if (!${$status{refaddr $self}} & QUEUED) { return; }
-        return if not defined $_client{refaddr $self};
-        return if ${$status{refaddr $self}} & CHECKING;
+        return if (${$status{refaddr $self}} & CHECKING);
+        return if !(${$status{refaddr $self}} & QUEUED);
         $downloaded{refaddr $self} += (($amount =~ m[^\d+$]) ? $amount : 0);
     }
 
-    sub _append_nodes {
-        my ($self, $nodes) = @_;
-        if (!${$status{refaddr $self}} & QUEUED) { return; }
-        return if not defined $_client{refaddr $self};
-        return if !$nodes;
-        $_nodes{refaddr $self} ||= q[];
-        return $_nodes{refaddr $self}
-            = compact(uncompact($_nodes{refaddr $self} . $nodes));
-    }
-
     sub _new_peer {
         my ($self) = @_;
         return if not defined $_client{refaddr $self};
-        return if ${$status{refaddr $self}} & CHECKING;
-        return if !${$status{refaddr $self}} & STARTED;
         $_client{refaddr $self}->_schedule(
-                                         {Time   => time + 5,
-                                          Code   => sub { shift->_new_peer },
-                                          Object => $self
-                                         }
+                             {Time => time + ($self->is_complete ? 60 : 5),
+                              Code => sub { shift->_new_peer if @_; },
+                              Object => $self
+                             }
         );
-        if (scalar(
-                grep {
-                    $_->{q[Object]}->isa(q[Net::BitTorrent::Peer])
-                        and not defined $_->{q[Object]}->peerid
-                    } values %{$_client{refaddr $self}->_connections}
-            ) >= 8
-            )
-        {   return;
+        return if (${$status{refaddr $self}} & CHECKING);
+        return if !(${$status{refaddr $self}} & STARTED);
+        return if !(${$status{refaddr $self}} & QUEUED);
+
+        # Don't bother if we're at the hard limit
+        return
+            if scalar $self->peers
+                >= $_client{refaddr $self}->_peers_per_torrent;
+
+        #
+        my $half_open = scalar(
+            grep {
+                $_->{q[Object]}->isa(q[Net::BitTorrent::Peer])
+                    and not defined $_->{q[Object]}->peerid
+                } values %{$_client{refaddr $self}->_connections}
+        );
+
+        #warn sprintf q[%d half open peers], $half_open;
+        # List of peers to make sure we're not already connected to this peer
+        my @peers = $self->peers;
+
+        # If we haven't any nodes in cache, gather them from various sources
+        if (!$_nodes{refaddr $self}) {
+            $_nodes{refaddr $self}
+                = $_client{refaddr $self}->_dht->_peers($self->infohash)
+                if !$self->private;
+            for my $tier (@{$trackers{refaddr $self}}) {
+                for my $url (@{$tier->urls}) {
+                    $_nodes{refaddr $self} .= $url->_peers;
+                }
+            }
         }
-        if ($self->is_complete)         { return; }
-        if (not $_nodes{refaddr $self}) { return; }
+
+        # Don't bother if we haven't any nodes to try
+        return if !$_nodes{refaddr $self};
+
+        # Inflate the list and try them one-by-one
         my @nodes = uncompact($_nodes{refaddr $self});
-        for (1 .. ($_client{refaddr $self}->_peers_per_torrent
-                       - scalar $self->_peers
-             )
-            )
-        {   last if not @nodes;
+        for ($half_open .. $_client{refaddr $self}->_half_open - 1) {
+            last if !@nodes;
             my $node = shift @nodes;
-            my $ok   = $_client{refaddr $self}
+            next
+                if scalar grep {
+                sprintf(q[%s:%d], ($_->_host || q[]), ($_->_port || 0)) eq
+                    $node    # already connected to this peer
+                } @peers;
+            my $ok = $_client{refaddr $self}
                 ->_event(q[ip_filter], {Address => $node});
             if (defined $ok and $ok == 0) { next; }
             my $peer =
                 Net::BitTorrent::Peer->new({Address => $node,
-                                            Torrent => $self
+                                            Torrent => $self,
+                                            Source  => q[TODO]
                                            }
                 );
-            last
-                if scalar(
-                grep {
-                    $_->{q[Object]}->isa(q[Net::BitTorrent::Peer])
-                        and not defined $_->{q[Object]}->peerid
-                    } values %{$_client{refaddr $self}->_connections}
-                ) >= $_client{refaddr $self}->_half_open;
         }
-        return 1;
-    }
 
-    sub _peers {
-        my ($self) = @_;
-        return if not defined $_client{refaddr $self};
-        return if !${$status{refaddr $self}} & QUEUED;
-        my $_connections = $_client{refaddr $self}->_connections;
-        return map {
-            (    ($_->{q[Object]}->isa(q[Net::BitTorrent::Peer]))
-             and ($_->{q[Object]}->_torrent)
-             and ($_->{q[Object]}->_torrent eq $self))
-                ? $_->{q[Object]}
-                : ()
-        } values %$_connections;
+        # Store only nodes we haven't tried yet
+        $_nodes{refaddr $self} = compact(@nodes);
+
+        # Return
+        return 1;
     }
 
     sub _add_tracker {
         my ($self, $tier) = @_;
-        return if not defined $_client{refaddr $self};
-        return if !${$status{refaddr $self}} & QUEUED;
         carp q[Please, pass new tier in an array ref...]
             unless ref $tier eq q[ARRAY];
-        return
-            push(@{$trackers{refaddr $self}},
-                 Net::BitTorrent::Torrent::Tracker->new(
-                                             {Torrent => $self, URLs => $tier}
-                 )
-            );
+        my $tracker = Net::BitTorrent::Torrent::Tracker->new(
+                                           {Torrent => $self, URLs => $tier});
+        $tracker->_announce(q[started]);
+        return push(@{$trackers{refaddr $self}}, $tracker);
     }
 
     sub _piece_by_index {
         my ($self, $index) = @_;
         return if !${$status{refaddr $self}} & STARTED;
+        return if (${$status{refaddr $self}} & CHECKING);
+        return if !(${$status{refaddr $self}} & QUEUED);
         if ((!defined $index) || ($index !~ m[^\d+$])) {
             carp
                 q[Net::BitTorrent::Torrent->_piece_by_index() requires an index];
@@ -654,6 +672,9 @@ package Net::BitTorrent::Torrent;
     sub _pick_piece {
         my ($self, $peer) = @_;
         return if $self->is_complete;
+        return if !${$status{refaddr $self}} & STARTED;
+        return if (${$status{refaddr $self}} & CHECKING);
+        return if !(${$status{refaddr $self}} & QUEUED);
         if (!$_client{refaddr $self}) {
             carp
                 q[Net::BitTorrent::Torrent->_pick_piece(PEER) will not on an orphan torrent];
@@ -830,9 +851,9 @@ package Net::BitTorrent::Torrent;
 
     sub _write_data {
         my ($self, $index, $offset, $data) = @_;
-        return if not defined $_client{refaddr $self};
-        return if ${$status{refaddr $self}} & CHECKING;
         return if !${$status{refaddr $self}} & STARTED;
+        return if (${$status{refaddr $self}} & CHECKING);
+        return if !(${$status{refaddr $self}} & QUEUED);
         if ((length($$data) + (
                  ($raw_data{refaddr $self}{q[info]}{q[piece length]} * $index)
                  + $offset
@@ -963,38 +984,42 @@ package Net::BitTorrent::Torrent;
     # Methods | Private | DHT
     sub _dht_announce {
         my ($self) = @_;
-        return if !${$status{refaddr $self}} & STARTED;
-        return if $self->private;
         $_client{refaddr $self}->_schedule(
                                      {Time   => time + 120,
                                       Code   => sub { shift->_dht_announce },
                                       Object => $self
                                      }
         );
-        if ($_client{refaddr $self}->_use_dht) {
-            $_client{refaddr $self}->_dht->_announce($self);
-            $_client{refaddr $self}->_schedule(
-                {   Time => time + 20,
-                    Code => sub {
-                        my ($s) = @_;
-                        $_client{refaddr $s}->_dht->_scrape($s)
-                            if $_client{refaddr $s}->_use_dht;
-                    },
-                    Object => $self
-                }
-            );
-        }
+        return if !${$status{refaddr $self}} & STARTED;
+        return if (${$status{refaddr $self}} & CHECKING);
+        return if !(${$status{refaddr $self}} & QUEUED);
+        return if $self->private;
+        return if !$_client{refaddr $self}->_use_dht;
+        $_client{refaddr $self}->_dht->_announce($self);
+        $_client{refaddr $self}->_schedule(
+            {   Time => time + 15,
+                Code => sub {
+                    my ($s) = @_;
+                    $_client{refaddr $s}->_dht->_scrape($s)
+                        if $_client{refaddr $s}->_use_dht;
+                },
+                Object => $self
+            }
+        );
     }
 
     sub _dht_scrape {
         my ($self) = @_;
-        return if $self->private;
         $_client{refaddr $self}->_schedule(
                                        {Time   => time + 60,
                                         Code   => sub { shift->_dht_scrape },
                                         Object => $self
                                        }
         );
+        return if !(${$status{refaddr $self}} & STARTED);
+        return if (${$status{refaddr $self}} & CHECKING);
+        return if !(${$status{refaddr $self}} & QUEUED);
+        return if $self->private;
         $_client{refaddr $self}->_dht->_scrape($self)
             if $_client{refaddr $self}->_use_dht;
     }
@@ -1036,15 +1061,34 @@ package Net::BitTorrent::Torrent;
     }
 
     # Methods | Public | Alpha
-    sub resume_data {
-        my ($self, $raw) = @_;
+    sub save_resume_data {
+        my ($self, $file) = @_;
+        $file ||= $resume_path{refaddr $self};
+        return if !$file;    # Don't even bother without a file to write to
 
         # Make sure file handles are closed so we don't mess up 'mtime' times
         for my $_file (@{$files{refaddr $self}}) { $_file->_close }
-        my %resume = %{$raw_data{refaddr $self}};
-        $resume{q[net-bittorrent]} = {
+
+        # Gather nodes from various sources
+        #   Internal
+        my $_nodes = $_nodes{refaddr $self};
+
+        #   DHT
+        $_nodes .= (((${$status{refaddr $self}} & QUEUED) && !$self->private)
+                    ? $_client{refaddr $self}->_dht->_peers($self->infohash)
+                    : q[]
+        );
+
+        #   Trackers
+        for my $tier (@{$trackers{refaddr $self}}) {
+            for my $url (@{$tier->urls}) { $_nodes .= $url->_peers; }
+        }
+
+        # The resume data proper
+        my %resume_data = (
+            q[.format]  => q[Net::BitTorrent resume],
             q[.t]       => time,
-            q[.version] => 1,
+            q[.version] => 2,
             bitfield    => ${$bitfield{refaddr $self}},
             files       => [
                 map {
@@ -1053,7 +1097,7 @@ package Net::BitTorrent::Torrent;
                     }
                     } @{$files{refaddr $self}}
             ],
-            peers   => $_nodes{refaddr $self},
+            peers => ($_nodes ? $_nodes : q[]),
             working => [
                 map {
                     {Block_Count => $_->{q[Block_Count]},
@@ -1069,8 +1113,12 @@ package Net::BitTorrent::Torrent;
                     }
                     } values %{$_working_pieces{refaddr $self}}
             ]
-        };
-        return $raw ? \%resume : bencode \%resume;
+        );
+
+        # Write it to disk
+        open(my ($_RD), q[>], $file) || return;
+        syswrite($_RD, bencode(\%resume_data)) || return;
+        return close $_RD;
     }
 
     # Methods | Public | Utility
@@ -1103,19 +1151,7 @@ Working: %s
 END
             $self->path, $raw_data{refaddr $self}{q[info]}{q[name]},
             $self->infohash(), $_basedir{refaddr $self}, $size{refaddr $self},
-            ${$status{refaddr $self}}, sub {
-            my ($s) = @_;
-            return ucfirst join q[, ],
-                grep {$_} ($s & LOADED) ? q[was loaded okay] : q[],
-                ($s & STARTED)           ? q[is started]                : q[],
-                ($s & CHECKING)          ? q[is currently hashchecking] : q[],
-                ($s & START_AFTER_CHECK) ? q[needs hashchecking]        : q[],
-                ($s & CHECKED)           ? q[has been checked]          : q[],
-                ($s & PAUSED)            ? q[has been paused]           : q[],
-                ($s & QUEUED) ? q[] : q[good for informational use only],
-                ($s & ERROR) ? q[but has an error] : q[];
-            }
-            ->(${$status{refaddr $self}}),
+            ${$status{refaddr $self}}, $self->_status_as_string(),
             ($self->private ? q[Disabled [Private]] : q[Enabled.]),
             100 - (grep {$_} split //,
                    unpack(q[b*], $wanted) / $self->piece_count * 100
@@ -1169,11 +1205,32 @@ END
             scalar @{$trackers{refaddr $self}},
             @{$trackers{refaddr $self}} != 1 ? q[s] : q[],
             join(qq[\n  ],
-                 map { $_->urls->[0]->url } @{$trackers{refaddr $self}}
+                 map     { $_->url }
+                     map { @{$_->urls} } @{$trackers{refaddr $self}}
             );
         return defined wantarray ? $dump : print STDERR qq[$dump\n];
     }
 
+    sub _status_as_string {
+        my ($self) = @_;
+        return ucfirst join q[, ],
+            grep {$_}
+            (${$status{refaddr $self}} & LOADED) ? q[was loaded okay] : q[],
+            (${$status{refaddr $self}} & STARTED) ? q[is started]
+            : q[is stopped],
+            (${$status{refaddr $self}} & CHECKING)
+            ? q[is currently hashchecking]
+            : q[],
+            (${$status{refaddr $self}} & START_AFTER_CHECK)
+            ? q[needs hashchecking]
+            : q[], (${$status{refaddr $self}} & CHECKED) ? q[has been checked]
+            : q[has not been checked],
+            (${$status{refaddr $self}} & PAUSED) ? q[has been paused] : q[],
+            (${$status{refaddr $self}} & QUEUED) ? q[is queued]
+            : q[is good for informational use only],
+            (${$status{refaddr $self}} & ERROR) ? q[but has an error] : q[];
+    }
+
     sub CLONE {
         for my $_oID (keys %REGISTRY) {
             my $_obj = $REGISTRY{$_oID};
@@ -1237,7 +1294,7 @@ L<queue ( )|/"queue ( CLIENT )">.
 
 Creates a C<Net::BitTorrent::Torrent> object.  This constructor is
 called by
-L<Net::BitTorrent::add_torrent( )|Net::BitTorrent/add_torrent ( { ... } )>.
+L<Net::BitTorrent->add_torrent( )|Net::BitTorrent/add_torrent ( { ... } )>.
 
 C<new( )> accepts arguments as a hash, using key-value pairs:
 
@@ -1270,6 +1327,17 @@ Filename of the .torrent file to load.
 
 This is the only required parameter.
 
+=item C<Resume>
+
+The filename used to gather and store resume data.
+
+This is an optional parameter.
+
+No default.  Without a defined resume file, resume data will not be
+written on calls to
+L<save_resume_data ( )|/"save_resume_data ( [ PATH ] )"> without a
+C<PATH> parameter.
+
 =item C<Status>
 
 Initial status of the torrent.  This parameter is ORed with the loaded
@@ -1380,6 +1448,18 @@ See the L<Events|/Events> section for more.
 
 Returns the L<filename|/"Path"> of the torrent this object represents.
 
+=item C<pause ( )>
+
+Pauses an active torrent without closing related sockets.
+
+See also: L<status ( )|/"status ( )">, L<stop ( )|/"stop ( )">,
+L<start ( )|/"start ( )">
+
+=item C<peers ( )>
+
+Returns a list of remote L<peers|Net::BitTorrent::Peer> related to this
+torrent.
+
 =item C<piece_count ( )>
 
 The number of pieces this torrent's data is broken into.
@@ -1405,13 +1485,17 @@ returns the original metadata in either bencoded form or as a raw hash
 (if you have other plans for the data) depending on the boolean value of
 the optional C<RAW> parameter.
 
-=item C<resume_data ( [ RAW ] )>
+=item C<resume_path ( )>
 
-One end of Net::BitTorrent's resume system.  This method returns the
-data as specified in
-L<Net::BitTorrent::Notes|Net::BitTorrent::Notes/"Resume API"> in either
-bencoded form or as a raw hash (if you have other plans for the data)
-depending on the boolean value of the optional C<RAW> parameter.
+Returns the default path used to
+L<store resume data|/"save_resume_data ( [ PATH ] )">.  This value is set
+in the C<Resume> parameter to L<new|/"new ( { [ARGS] } )">.
+
+=item C<save_resume_data ( [ PATH ] )>
+
+One end of Net::BitTorrent's resume system.  This method writes the
+data to the file specified in the call to L<new( )|/"new ( { [ARGS] } )">
+or (if defined) to the C<PATH> parameter.
 
 See also:
 L<Resume API|Net::BitTorrent::Notes/"Resume API">
@@ -1605,13 +1689,6 @@ disconnected and all files are closed.
 See also: L<status ( )|/"status ( )">, L<start ( )|/"start ( )">,
 L<pause ( )|/"pause ( )">
 
-=item C<pause ( )>
-
-Pauses an active torrent without closing related sockets.
-
-See also: L<status ( )|/"status ( )">, L<stop ( )|/"stop ( )">,
-L<start ( )|/"start ( )">
-
 =item C<trackers>
 
 Returns a list of all
@@ -1668,6 +1745,6 @@ clarification, see http://creativecommons.org/licenses/by-sa/3.0/us/.
 Neither this module nor the L<Author|/Author> is affiliated with
 BitTorrent, Inc.
 
-=for svn $Id: Torrent.pm 46 2008-12-30 23:25:17Z sanko@cpan.org $
+=for svn $Id: Torrent.pm 49 2009-01-05 22:38:02Z sanko@cpan.org $
 
 =cut
