@@ -8,13 +8,17 @@ package Net::BitTorrent::Torrent::Tracker::UDP;
     use List::Util qw[sum];
     use Socket qw[inet_aton pack_sockaddr_in];
     use lib q[../../../../../lib];
-    use Net::BitTorrent::Util qw[uncompact];
+    use Net::BitTorrent::Util qw[:compact];
     use version qw[qv];
-    our $SVN = q[$Id: UDP.pm 46 2008-12-30 23:25:17Z sanko@cpan.org $];
-    our $UNSTABLE_RELEASE = 3; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new((qw$Rev: 46 $)[1])->numify / 1000), $UNSTABLE_RELEASE);
+    our $SVN = q[$Id: UDP.pm 49 2009-01-05 22:38:02Z sanko@cpan.org $];
+    our $UNSTABLE_RELEASE = 2; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new((qw$Rev: 49 $)[1])->numify / 1000), $UNSTABLE_RELEASE);
     my %REGISTRY = ();
-    my @CONTENTS = \my (%_url, %_tier, %_tid, %_cid, %_outstanding_requests,
-                        %_packed_host, %_event);
+    my @CONTENTS = \my (%_url,                  %_tier,
+                        %_tid,                  %_cid,
+                        %_outstanding_requests, %_packed_host,
+                        %_event,                %_peers,
+                        %_complete,             %_incomplete
+    );
 
     sub new {
         my ($class, $args) = @_;
@@ -46,10 +50,13 @@ package Net::BitTorrent::Torrent::Tracker::UDP;
         else { $packed_host = inet_aton($host) }
         $_packed_host{refaddr $self}
             = pack_sockaddr_in($port, inet_aton($host));
-        $_url{refaddr $self}   = $args->{q[URL]};
-        $_event{refaddr $self} = q[];
-        $_tier{refaddr $self}  = $args->{q[Tier]};
-        $_tid{refaddr $self}   = int(rand() * 26**5);
+        $_url{refaddr $self}        = $args->{q[URL]};
+        $_event{refaddr $self}      = q[];
+        $_tier{refaddr $self}       = $args->{q[Tier]};
+        $_peers{refaddr $self}      = q[];
+        $_complete{refaddr $self}   = 0;
+        $_incomplete{refaddr $self} = 0;
+        $_tid{refaddr $self}        = int(rand() * 26**5);
         weaken $_tier{refaddr $self};
         weaken($REGISTRY{refaddr $self} = $self);
         return $self;
@@ -61,6 +68,7 @@ package Net::BitTorrent::Torrent::Tracker::UDP;
     # Accessors | Private
     sub _packed_host { return $_packed_host{refaddr +shift} }
     sub _tier        { return $_tier{refaddr +shift}; }
+    sub _peers       { return $_peers{refaddr +shift}; }
     sub _client      { return $_tier{refaddr +shift}->_client }
 
     # Methods | Private
@@ -206,9 +214,9 @@ package Net::BitTorrent::Torrent::Tracker::UDP;
             if (length($data) >= 20) {
                 my ($min_interval, $leeches, $seeds, $peers)
                     = unpack(q[N N N a*], $packet);
-                $_tier{refaddr $self}->_torrent->_append_nodes($peers);
-                $_tier{refaddr $self}->_set_complete($seeds);
-                $_tier{refaddr $self}->_set_incomplete($leeches);
+                $_peers{refaddr $self}      = $peers;
+                $_complete{refaddr $self}   = $seeds;
+                $_incomplete{refaddr $self} = $leeches;
                 $_tier{refaddr $self}->_torrent->_event(
                                             q[tracker_success],
                                             {Tracker => $self,
@@ -406,6 +414,6 @@ clarification, see http://creativecommons.org/licenses/by-sa/3.0/us/.
 Neither this module nor the L<Author|/Author> is affiliated with
 BitTorrent, Inc.
 
-=for svn $Id: UDP.pm 46 2008-12-30 23:25:17Z sanko@cpan.org $
+=for svn $Id: UDP.pm 49 2009-01-05 22:38:02Z sanko@cpan.org $
 
 =cut
