@@ -3,17 +3,17 @@ use strict;
 use warnings;
 use Data::Dump;
 use Net::BitTorrent::Torrent;
-my $torrent = Net::BitTorrent::Torrent->new({Path => '956_dht.torrent'})
+my $torrent = Net::BitTorrent::Torrent->new({Path => 'a.legal.torrent'})
     or exit;
 $torrent->as_string(1);
-dd $torrent->metadata(1);
+dd $torrent->raw_data(1);
 print map { qq[\n] . $_->path } @{$torrent->files};
 
 =pod
 
 =head1 NAME
 
-001-torrent-info.pl - Demonstration of using a standalone Net::BitTorrent::Torrent object to gather information
+/tatoeba/001-torrent-info.pl - Use Net::BitTorrent to gather information
 
 =head1 Description
 
@@ -27,8 +27,6 @@ file.
 
 =head1 Lowdown
 
-This section only makes sense when you view the source.
-
 =over
 
 =item Line 6
@@ -51,7 +49,7 @@ to give you a rundown of what can be parsed from the file.
 =item Line 9
 
 Prints a dump of the .torrent file's
-L<metadata|Net::BitTorrent::Torrent/"metadata ( [ RAW ] )">.
+L<metadata|Net::BitTorrent::Torrent/"raw_data ( [ RAW ] )">.
 
 =item Line 10
 
@@ -75,7 +73,7 @@ CPAN ID: SANKO
 
 =head1 License and Legal
 
-Copyright (C) 2008-2009 by Sanko Robinson <sanko@cpan.org>
+Copyright (C) 2008 by Sanko Robinson E<lt>sanko@cpan.orgE<gt>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of The Artistic License 2.0.  See the F<LICENSE>
@@ -91,6 +89,6 @@ clarification, see http://creativecommons.org/licenses/by-sa/3.0/us/.
 Neither this module nor the L<Author|/Author> is affiliated with
 BitTorrent, Inc.
 
-=for svn $Id: 001-torrent-info.pl c260f8c 2009-09-07 02:35:29Z sanko@cpan.org $
+=for svn $Id: 001-torrent-info.pl 46 2008-12-30 23:25:17Z sanko@cpan.org $
 
 =cut
