@@ -36,7 +36,7 @@ if (!$torrent) {
     exit;
 }
 l q[Loaded torrent okay. Raw data follows...];
-l Dumper $torrent->metadata(1);
+l Dumper $torrent->raw_data(1);
 l q[Setting client-wide callbacks...];
 $client->on_event(
     q[ip_filter],
@@ -162,7 +162,7 @@ l q[Exiting];
 
 =head1 NAME
 
-002-debug.pl - Demonstration script that logs EVERYTHING
+/tatoeba/002-debug.pl - Logs EVERYTHING to a file for debugging
 
 =head1 Description
 
@@ -182,7 +182,7 @@ CPAN ID: SANKO
 
 =head1 License and Legal
 
-Copyright (C) 2008-2009 by Sanko Robinson <sanko@cpan.org>
+Copyright (C) 2008 by Sanko Robinson E<lt>sanko@cpan.orgE<gt>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of The Artistic License 2.0.  See the F<LICENSE>
@@ -198,6 +198,6 @@ clarification, see http://creativecommons.org/licenses/by-sa/3.0/us/.
 Neither this module nor the L<Author|/Author> is affiliated with
 BitTorrent, Inc.
 
-=for svn $Id: 002-debug.pl c260f8c 2009-09-07 02:35:29Z sanko@cpan.org $
+=for svn $Id: 002-debug.pl 49 2009-01-05 22:38:02Z sanko@cpan.org $
 
 =cut
