@@ -23,7 +23,7 @@ sleep(0.25) and $client->do_one_loop(0.25) while !$torrent->is_complete;
 
 =head1 NAME
 
-003-threads.pl - Trivial Demonstration of a Multi-threaded Client
+/tatoeba/003-threads.pl - Trivial, Multi-threaded Example
 
 =head1 Description
 
@@ -38,8 +38,6 @@ dark matter or causes you to foam at the mouth.
 
 =head1 Lowdown
 
-This section only makes sense when you view the source.
-
 =over
 
 =item Line 5-6
@@ -72,7 +70,7 @@ Validates data. As this starts,
 connections to L<peers|Net::BitTorrent::Torrent/"peers ( )"> related to
 this L<torrent|Net::BitTorrent::Torrent> in the parent thread are closed.
 While the child does the checking, our
-L<bitfield|Net::BitTorrent::Torrent/"bitfield ( )"> is kept in sync with
+L<bitfield|Net::BitTorrent::Torrent/"bitfield ( )"> is kept in synch with
 the parent (thanks to L<threads::shared|threads::shared>), and our
 L<status|Net::BitTorrent::Torrent/"status ( )"> goes through some
 changes.
@@ -106,7 +104,7 @@ CPAN ID: SANKO
 
 =head1 License and Legal
 
-Copyright (C) 2008-2009 by Sanko Robinson <sanko@cpan.org>
+Copyright (C) 2008 by Sanko Robinson E<lt>sanko@cpan.orgE<gt>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of The Artistic License 2.0.  See the F<LICENSE>
@@ -122,6 +120,6 @@ clarification, see http://creativecommons.org/licenses/by-sa/3.0/us/.
 Neither this module nor the L<Author|/Author> is affiliated with
 BitTorrent, Inc.
 
-=for svn $Id: 003-threads.pl 3685ae3 2009-05-03 20:38:28Z sanko@cpan.org $
+=for svn $Id: 003-threads.pl 48 2009-01-02 17:39:49Z sanko@cpan.org $
 
 =cut
